@@ -2,7 +2,7 @@ import sys
 from contextlib import contextmanager
 from datetime import datetime
 
-from asn1crypto import pem
+from asn1crypto import pem, ocsp, algos
 from dateutil.parser import parse as parse_dt
 
 import click
@@ -11,7 +11,7 @@ import logging
 
 from .config_utils import ConfigurationError
 from .crypto_utils import pyca_cryptography_present
-from .registry import CertomancerConfig, CertLabel
+from .registry import CertomancerConfig, CertLabel, ServiceLabel
 from .services import CertomancerServiceError
 from .version import __version__
 
@@ -145,6 +145,8 @@ def mass_summon(ctx, architecture, output, no_pem, archive, flat, no_pfx,
 @click.argument('architecture', type=str, metavar='PKI_ARCH')
 @click.argument('cert_label', type=click.Path(writable=True), required=True)
 @click.argument('output', type=click.Path(writable=True), required=False)
+@click.option('--attr', type=bool, is_flag=True,
+              help='fetch an attribute certificate instead of a regular one')
 @click.option('--ignore-tty', type=bool, is_flag=True,
               help='never try to prevent binary data from being written '
                    'to stdout')
@@ -155,8 +157,8 @@ def mass_summon(ctx, architecture, output, no_pem, archive, flat, no_pfx,
 @click.option('--no-pem', help='use raw DER instead of PEM output',
               required=False, type=bool, is_flag=True)
 @exception_manager()
-def summon(ctx, architecture, cert_label, output, no_pem, as_pfx, ignore_tty,
-           pfx_pass):
+def summon(ctx, architecture, attr, cert_label, output, no_pem, as_pfx,
+           ignore_tty, pfx_pass):
     cfg: CertomancerConfig = next(ctx.obj['config'])
     pki_arch = cfg.get_pki_arch(architecture)
     if as_pfx and not pyca_cryptography_present():
@@ -175,13 +177,20 @@ def summon(ctx, architecture, cert_label, output, no_pem, as_pfx, ignore_tty,
         )
 
     if as_pfx:
+        if attr:
+            raise click.ClickException(
+                "Attribute certificates are not supported in PKCS#12 output"
+            )
         if pfx_pass is not None:
             pfx_pass = pfx_pass.encode('utf8')
         data = pki_arch.package_pkcs12(cert_label, password=pfx_pass)
     else:
-        data = pki_arch.get_cert(CertLabel(cert_label)).dump()
+        if attr:
+            data = pki_arch.get_attr_cert(CertLabel(cert_label)).dump()
+        else:
+            data = pki_arch.get_cert(CertLabel(cert_label)).dump()
         if not no_pem:
-            data = pem.armor('certificate', data)
+            data = pem.armor(f'{"attribute " if attr else ""}certificate', data)
 
     if output is None:
         # we want to write bytes, not strings
@@ -215,7 +224,7 @@ def necronomicon(ctx, architecture, crl_repo, output, no_pem, at_time,
     else:
         at_time = parse_dt(at_time)
     crl = pki_arch.service_registry.get_crl(
-        repo_label=crl_repo, at_time=at_time
+        repo_label=ServiceLabel(crl_repo), at_time=at_time
     )
 
     if output is None and no_pem and not ignore_tty and sys.stdout.isatty():
@@ -234,6 +243,73 @@ def necronomicon(ctx, architecture, crl_repo, output, no_pem, at_time,
             f.write(data)
 
 
+@cli.command(help='query an OCSP responder')
+@click.pass_context
+@click.argument('architecture', type=str, metavar='PKI_ARCH')
+@click.argument('cert_label', type=str, metavar='CERT_LABEL')
+@click.argument('responder', type=str, metavar='OCSP_RESPONDER')
+@click.argument('output', type=click.Path(writable=True, dir_okay=False),
+                required=False)
+@click.option('--ignore-tty', type=bool, is_flag=True,
+              help='never try to prevent binary data from being written '
+                   'to stdout')
+@click.option('--at-time', required=False, type=str,
+              help=('ISO 8601 timestamp at which to evaluate '
+                    'revocation status [default: now]'))
+@exception_manager()
+def seance(ctx, architecture, cert_label, responder, output,
+           ignore_tty, at_time):
+    cfg: CertomancerConfig = next(ctx.obj['config'])
+    pki_arch = cfg.get_pki_arch(architecture)
+    if at_time is None:
+        at_time = datetime.now(tz=tzlocal.get_localzone())
+    else:
+        at_time = parse_dt(at_time)
+
+    # Format CertId value based on certificate spec (or attr cert spec)
+    # We differentiate between attr certs and regular PKCs based on the
+    # responder's service info settings.
+    svc_info = pki_arch.service_registry.get_ocsp_info(ServiceLabel(responder))
+    if svc_info.is_aa_responder:
+        cert_spec = pki_arch.get_attr_cert_spec(CertLabel(cert_label))
+    else:
+        cert_spec = pki_arch.get_cert_spec(CertLabel(cert_label))
+    issuer_cert_label = cert_spec.resolve_issuer_cert(pki_arch)
+    issuer_cert = pki_arch.get_cert(issuer_cert_label)
+    cert_id = ocsp.CertId({
+        'hash_algorithm': algos.DigestAlgorithm(
+            {'algorithm': 'sha256'}
+        ),
+        'issuer_name_hash': pki_arch.entities.get_name_hash(
+            cert_spec.issuer, 'sha256'
+        ),
+        'issuer_key_hash': issuer_cert.public_key.sha256,
+        'serial_number': cert_spec.serial,
+    })
+
+    # Initialise the requested OCSP responder
+    ocsp_responder = pki_arch.service_registry.summon_responder(
+        label=ServiceLabel(responder), at_time=at_time
+    )
+    sing_resp = ocsp_responder.format_single_ocsp_response(
+        cid=cert_id, issuer_cert=issuer_cert
+    )
+    response = \
+        ocsp_responder.assemble_simple_ocsp_responses(responses=[sing_resp])
+
+    if output is None and not ignore_tty and sys.stdout.isatty():
+        raise click.ClickException(
+            "Refusing to write binary output to a TTY. Pass --ignore-tty if "
+            "you really want to ignore this check."
+        )
+    data = response.dump()
+    if output is None:
+        sys.stdout.buffer.write(data)
+    else:
+        with open(output, 'wb') as f:
+            f.write(data)
+
+
 @cli.command(help='run the Animator behind a development server')
 @click.option('--port', help='port to listen on',
               required=False, type=int, default=9000, show_default=True)
